"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tape_1 = __importDefault(require("tape"));
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("./utils");
const amman_1 = require("@metaplex-foundation/amman");
const utils_2 = require("../src/utils");
const transactions_1 = require("./transactions");
const utils_3 = require("./utils");
const mint_nft_1 = require("./actions/mint-nft");
(0, utils_1.killStuckProcess)();
(0, tape_1.default)('create-market: success', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const payer = web3_js_1.Keypair.generate();
    (0, utils_3.addLabel)('create:payer', payer);
    const connection = new web3_js_1.Connection(utils_1.connectionURL, 'confirmed');
    const transactionHandler = new amman_1.PayerTransactionHandler(connection, payer);
    yield (0, amman_1.airdrop)(connection, payer.publicKey, 20);
    const { store, transaction: createStoreTx } = yield (0, transactions_1.createStoreTransaction)(payer, connection);
    (0, utils_3.logDebug)('STORE store: ', store.publicKey.toBase58());
    (0, utils_3.logDebug)('STORE payer: ', payer.publicKey.toBase58());
    const createStoreRes = yield transactionHandler.sendAndConfirmTransaction(createStoreTx, [store], amman_1.defaultSendOptions);
    (0, utils_3.addLabel)('create:store', store);
    (0, amman_1.assertConfirmedTransaction)(t, createStoreRes.txConfirmed);
    const { edition: masterEdition, editionBump: masterEditionBump, tokenAccount: resourceToken, mint: resourceMint, } = yield (0, mint_nft_1.mintNFT)({
        transactionHandler,
        payer: payer.publicKey,
        connection,
    });
    (0, utils_3.logDebug)(`SR masterEdition:     ${masterEdition.toBase58()}`);
    (0, utils_3.logDebug)(`SR masterEditionBump: ${masterEditionBump}`);
    (0, utils_3.logDebug)(`SR resourceToken:     ${resourceToken.publicKey.toBase58()}`);
    (0, utils_3.logDebug)(`SR resourceMint:      ${resourceMint.publicKey.toBase58()}`);
    const [vaultOwner, vaultOwnerBump] = yield (0, utils_2.findVaultOwnerAddress)(resourceMint.publicKey, store.publicKey);
    (0, utils_3.logDebug)('-- Vault Owner -------------------------------');
    (0, utils_3.logDebug)(`SR vaultOwner:        ${vaultOwner.toBase58()}`);
    (0, utils_3.logDebug)(`SR vaultOwnerBump:    ${vaultOwnerBump}`);
    const { tokenAccount: vault, createTokenTx: createVaultTx } = yield (0, transactions_1.createTokenAccount)({
        payer: payer.publicKey,
        mint: resourceMint.publicKey,
        connection,
        owner: vaultOwner,
    });
    (0, utils_3.logDebug)('-- Vault  -------------------------------');
    (0, utils_3.logDebug)(`SR vault:             ${vault.publicKey.toBase58()}`);
    (0, utils_3.logDebug)(`SR createVaultTx:     ${createVaultTx}`);
    const createVaultRes = yield transactionHandler.sendAndConfirmTransaction(createVaultTx, [vault], amman_1.defaultSendOptions);
    (0, utils_3.addLabel)('create:vault', vault);
    (0, amman_1.assertConfirmedTransaction)(t, createVaultRes.txConfirmed);
    const { initSellingResourceTx, sellingResource } = yield (0, transactions_1.createInitSellingResourceTransaction)({
        payer,
        connection,
        store,
        masterEdition,
        masterEditionBump,
        resourceMint: resourceMint.publicKey,
        resourceToken: resourceToken.publicKey,
        vault,
        owner: vaultOwner,
        vaultOwnerBump,
        maxSupply: 100,
    });
    (0, utils_3.logDebug)('-- SR  -------------------------------');
    (0, utils_3.logDebug)(`SR sellingResource:   ${sellingResource.publicKey.toBase58()}`);
    const initSellingResourceRes = yield transactionHandler.sendAndConfirmTransaction(initSellingResourceTx, [sellingResource, vault], amman_1.defaultSendOptions);
    (0, utils_3.addLabel)('create:selling-resource', sellingResource);
    (0, amman_1.assertConfirmedTransaction)(t, initSellingResourceRes.txConfirmed);
    (0, utils_3.logDebug)('-- initSellingResourceRes  --------------');
    (0, utils_3.logDebug)(initSellingResourceRes.txSummary.logMessages.join('\n'));
    const { mint: treasuryMint } = yield (0, mint_nft_1.mintNFT)({
        transactionHandler,
        payer: payer.publicKey,
        connection,
    });
    (0, utils_3.logDebug)('-- Market  -------------------------------');
    (0, utils_3.logDebug)(`Market treasuryMint:   ${treasuryMint.publicKey.toBase58()}`);
    const [treasuryOwner, treasuryOwnerBump] = yield (0, utils_2.findTresuryOwnerAddress)(treasuryMint.publicKey, sellingResource.publicKey);
    (0, utils_3.logDebug)(`Market treasuryOwner:       ${treasuryOwner.toBase58()}`);
    (0, utils_3.logDebug)(`Market treasuryOwnerBump:   ${treasuryOwnerBump}`);
    const { tokenAccount: treasuryHolder, createTokenTx: createTreasuryTx } = yield (0, transactions_1.createTokenAccount)({
        payer: payer.publicKey,
        connection,
        mint: treasuryMint.publicKey,
        owner: treasuryOwner,
    });
    (0, utils_3.logDebug)(`Market treasuryHolder:   ${treasuryHolder.publicKey.toBase58()}`);
    const createTreasuryRes = yield transactionHandler.sendAndConfirmTransaction(createTreasuryTx, [treasuryHolder], amman_1.defaultSendOptions);
    (0, utils_3.addLabel)('create:treasury', treasuryHolder);
    (0, amman_1.assertConfirmedTransaction)(t, createTreasuryRes.txConfirmed);
    const startDate = Math.round(Date.now() / 1000) + 5;
    const endDate = startDate + 5 * 20;
    const marketName = 'N';
    const marketDescription = 'D';
    const mutable = true;
    const price = 2;
    const piecesInOneWallet = 10;
    const { marketTx, market } = yield (0, transactions_1.createMarketTransaction)({
        payer,
        connection,
        store,
        sellingResourceOwner: payer,
        sellingResource: sellingResource.publicKey,
        mint: treasuryMint.publicKey,
        treasuryHolder: treasuryHolder.publicKey,
        owner: treasuryOwner,
        treasuryOwnerBump,
        name: marketName,
        description: marketDescription,
        mutable,
        price,
        piecesInOneWallet,
        startDate,
        endDate,
    });
    const marketRes = yield transactionHandler.sendAndConfirmTransaction(marketTx, [market, payer], amman_1.defaultSendOptions);
    (0, utils_3.addLabel)('create:market', market);
    (0, amman_1.assertConfirmedTransaction)(t, marketRes.txConfirmed);
    (0, utils_3.logDebug)(`market: ${market.publicKey}`);
    (0, utils_3.logDebug)(marketRes.txSummary.logMessages.join('\n'));
}));
//# sourceMappingURL=create-market.test.js.map